"""
Twitch OAuth2 backend, docs at:
    https://python-social-auth.readthedocs.io/en/latest/backends/twitch.html
"""
from .oauth import BaseOAuth2
from .open_id_connect import OpenIdConnectAuth


class TwitchOpenIdConnect(OpenIdConnectAuth):
    """Twitch OpenID Connect authentication backend"""
    name = 'twitch'
    USERNAME_KEY = 'preferred_username'
    OIDC_ENDPOINT = 'https://id.twitch.tv/oauth2'
    DEFAULT_SCOPE = ['openid', 'user:read:email']
    TWITCH_CLAIMS = '{"id_token":{"email": null,"email_verified":null,"preferred_username":null}}'

    def auth_params(self, state=None):
        params = super().auth_params(state)
        # Twitch uses a non-compliant OpenID implementation where the claims must be passed as a param
        params['claims'] = self.TWITCH_CLAIMS
        return params

    def get_user_details(self, response):
        return {
            'username': self.id_token['preferred_username'],
            'email': self.id_token['email'],
            # Twitch does not provide this information
            'fullname': '',
            'first_name': '',
            'last_name': '',
        }


class TwitchOAuth2(BaseOAuth2):
    """Twitch OAuth authentication backend"""
    name = 'twitch'
    ID_KEY = '_id'
    AUTHORIZATION_URL = 'https://id.twitch.tv/oauth2/authorize'
    ACCESS_TOKEN_URL = 'https://id.twitch.tv/oauth2/token'
    ACCESS_TOKEN_METHOD = 'POST'
    DEFAULT_SCOPE = ['user:read:email']
    REDIRECT_STATE = False

    def get_user_id(self, details, response):
        """
        Use twitch user id as unique id
        """
        return response.get('id')

    def get_user_details(self, response):
        return {
            'username': response.get('login'),
            'email': response.get('email'),
            'first_name': '',
            'last_name': ''
        }

    def user_data(self, access_token, *args, **kwargs):
        client_id, _ = self.get_key_and_secret()
        auth_headers = {
            'Authorization': 'Bearer %s' % access_token,
            'Client-Id': client_id
        }
        url = 'https://api.twitch.tv/helix/users'

        data = self.get_json(url, headers=auth_headers)

        return data['data'][0] if data.get('data') else {}



import json

from .oauth import OAuth2Test
from .test_open_id_connect import OpenIdConnectTestMixin


class TwitchOpenIdConnectTest(OpenIdConnectTestMixin, OAuth2Test):
    backend_path = 'social_core.backends.twitch.TwitchOpenIdConnect'
    user_data_url = 'https://id.twitch.tv/oauth2/userinfo'
    issuer = 'https://id.twitch.tv/oauth2'
    expected_username = 'test_user1'
    openid_config_body = json.dumps({
        'authorization_endpoint': 'https://id.twitch.tv/oauth2/authorize',
        'claims_parameter_supported': True,
        'claims_supported': [
            'iss',
            'azp',
            'preferred_username',
            'updated_at',
            'aud',
            'exp',
            'iat',
            'picture',
            'sub',
            'email',
            'email_verified',
        ],
        'id_token_signing_alg_values_supported': [
            'RS256',
        ],
        'issuer': 'https://id.twitch.tv/oauth2',
        'jwks_uri': 'https://id.twitch.tv/oauth2/keys',
        'response_types_supported': [
            'id_token',
            'code',
            'token',
            'code id_token',
            'token id_token',
        ],
        'scopes_supported': [
            'openid',
        ],
        'subject_types_supported': [
            'public',
        ],
        'token_endpoint': 'https://id.twitch.tv/oauth2/token',
        'token_endpoint_auth_methods_supported': [
            'client_secret_post',
        ],
        'userinfo_endpoint': 'https://id.twitch.tv/oauth2/userinfo',
    })


class TwitchOAuth2Test(OAuth2Test):
    backend_path = 'social_core.backends.twitch.TwitchOAuth2'
    user_data_url = 'https://api.twitch.tv/helix/users'
    expected_username = 'test_user1'
    access_token_body = json.dumps({
        'access_token': 'foobar',
        'token_type': 'bearer',
    })
    user_data_body = json.dumps({
        'data': [
            {
                'id': '689563726',
                'login': 'test_user1',
                'display_name': 'test_user1',
                'type': '',
                'broadcaster_type': '',
                'description': '',
                'profile_image_url': 'https://static-cdn.jtvnw.net/jtv_user_pictures/foo.png',
                'offline_image_url': '',
                'view_count': 0,
                'email': 'example@reply.com',
                'created_at': '2021-05-21T18:59:25Z',
                'access_token': 'hmkgz15x7j54jm63rpwfwhcnue6t4fxwv'
            }
        ]
    })

    def test_login(self):
        self.do_login()

    def test_partial_pipeline(self):
        self.do_partial_pipeline()



from .oauth import BaseOAuth2


class CILogonOAuth2(BaseOAuth2):
    """
    CI Logon Authentication Backend

    Docs: https://www.cilogon.org/oidc
    """

    name = 'cilogon-oauth2'
    AUTHORIZATION_URL = 'https://cilogon.org/authorize'
    ACCESS_TOKEN_URL = 'https://cilogon.org/oauth2/token'
    ACCESS_TOKEN_METHOD = 'POST'
    DEFAULT_SCOPE = ['openid', 'email', 'profile', 'org.cilogon.userinfo']
    REDIRECT_STATE = False
    SCOPE_SEPARATOR = '+'

    def user_data(self, token, *args, **kwargs):
        """Loads user data from endpoint"""
        url = 'https://cilogon.org/oauth2/userinfo'
        data = {'access_token': token}
        try:
            return self.get_json(url, method='POST', data=data)
        except ValueError:
            return None
    
    def get_user_id(self, details, response):
        """Return user unique id provided by service
           In this case it is a combination of the `sub`
           and `iss` respective values."""
        return response.get('sub', '') + ' ' + response.get('iss', '')

    def get_user_details(self, response):
        """Return user details from CI Logon service"""
        fullname, first_name, last_name = self.get_user_names(
            first_name=response.get('given_name'),
            last_name=response.get('family_name')
        )
        return {
            'username': response.get('email'),
            'email': response.get('email'),
            'fullname': fullname,
            'first_name': first_name,
            'last_name': last_name,
        }



import json
from urllib.parse import urlencode

from httpretty import HTTPretty

from ...exceptions import AuthForbidden
from .oauth import OAuth1Test, OAuth2Test


class BitbucketOAuthMixin:
    user_data_url = 'https://api.bitbucket.org/2.0/user'
    expected_username = 'foobar'
    bb_api_user_emails = 'https://api.bitbucket.org/2.0/user/emails'

    user_data_body = json.dumps({
        'created_on': '2012-03-29T18:07:38+00:00',
        'display_name': 'Foo Bar',
        'links': {
            'avatar': {'href': 'https://bitbucket.org/account/foobar/avatar/32/'},
            'followers': {'href': 'https://api.bitbucket.org/2.0/users/foobar/followers'},
            'following': {'href': 'https://api.bitbucket.org/2.0/users/foobar/following'},
            'hooks': {'href': 'https://api.bitbucket.org/2.0/users/foobar/hooks'},
            'html': {'href': 'https://bitbucket.org/foobar'},
            'repositories': {'href': 'https://api.bitbucket.org/2.0/repositories/foobar'},
            'self': {'href': 'https://api.bitbucket.org/2.0/users/foobar'}},
        'location': 'Fooville, Bar',
        'type': 'user',
        'username': 'foobar',
        'uuid': '{397621dc-0f78-329f-8d6d-727396248e3f}',
        'website': 'http://foobar.com'
    })

    emails_body = json.dumps({
        'page': 1,
        'pagelen': 10,
        'size': 2,
        'values': [
            {
                'email': 'foo@bar.com',
                'is_confirmed': True,
                'is_primary': True,
                'links': {'self': {'href': 'https://api.bitbucket.org/2.0/user/emails/foo@bar.com'}},
                'type': 'email'
            },
            {
                'email': 'not@confirme.com',
                'is_confirmed': False,
                'is_primary': False,
                'links': {'self': {'href': 'https://api.bitbucket.org/2.0/user/emails/not@confirmed.com'}},
                'type': 'email'
            }
        ]
    })


class BitbucketOAuth1Test(BitbucketOAuthMixin, OAuth1Test):
    backend_path = 'social_core.backends.bitbucket.BitbucketOAuth'

    request_token_body = urlencode({
        'oauth_token_secret': 'foobar-secret',
        'oauth_token': 'foobar',
        'oauth_callback_confirmed': 'true'
    })

    access_token_body = json.dumps({
        'access_token': 'foobar',
        'token_type': 'bearer'
    })

    def test_login(self):
        HTTPretty.register_uri(HTTPretty.GET,
                               self.bb_api_user_emails,
                               status=200, body=self.emails_body)
        self.do_login()

    def test_partial_pipeline(self):
        HTTPretty.register_uri(HTTPretty.GET,
                               self.bb_api_user_emails,
                               status=200, body=self.emails_body)
        self.do_partial_pipeline()


class BitbucketOAuth1FailTest(BitbucketOAuth1Test):
    emails_body = json.dumps({
        'page': 1,
        'pagelen': 10,
        'size': 1,
        'values': [
            {
                'email': 'foo@bar.com',
                'is_confirmed': False,
                'is_primary': True,
                'links': {'self': {'href': 'https://api.bitbucket.org/2.0/user/emails/foo@bar.com'}},
                'type': 'email'
            }
        ]
    })

    def test_login(self):
        self.strategy.set_settings({
            'SOCIAL_AUTH_BITBUCKET_VERIFIED_EMAILS_ONLY': True
        })
        with self.assertRaises(AuthForbidden):
            super().test_login()

    def test_partial_pipeline(self):
        self.strategy.set_settings({
            'SOCIAL_AUTH_BITBUCKET_VERIFIED_EMAILS_ONLY': True
        })
        with self.assertRaises(AuthForbidden):
            super().test_partial_pipeline()


class BitbucketOAuth2Test(BitbucketOAuthMixin, OAuth2Test):
    backend_path = 'social_core.backends.bitbucket.BitbucketOAuth2'

    access_token_body = json.dumps({
        'access_token': 'foobar_access',
        'scopes': 'foo_scope',
        'expires_in': 3600,
        'refresh_token': 'foobar_refresh',
        'token_type': 'bearer'
    })

    def test_login(self):
        HTTPretty.register_uri(HTTPretty.GET,
                               self.bb_api_user_emails,
                               status=200, body=self.emails_body)
        self.do_login()

    def test_partial_pipeline(self):
        HTTPretty.register_uri(HTTPretty.GET,
                               self.bb_api_user_emails,
                               status=200, body=self.emails_body)
        self.do_partial_pipeline()


class BitbucketOAuth2FailTest(BitbucketOAuth2Test):
    emails_body = json.dumps({
        'page': 1,
        'pagelen': 10,
        'size': 1,
        'values': [
            {
                'email': 'foo@bar.com',
                'is_confirmed': False,
                'is_primary': True,
                'links': {'self': {'href': 'https://api.bitbucket.org/2.0/user/emails/foo@bar.com'}},
                'type': 'email'
            }
        ]
    })

    def test_login(self):
        self.strategy.set_settings({
            'SOCIAL_AUTH_BITBUCKET_OAUTH2_VERIFIED_EMAILS_ONLY': True
        })
        with self.assertRaises(AuthForbidden):
            super().test_login()

    def test_partial_pipeline(self):
        self.strategy.set_settings({
            'SOCIAL_AUTH_BITBUCKET_OAUTH2_VERIFIED_EMAILS_ONLY': True
        })
        with self.assertRaises(AuthForbidden):
            super().test_partial_pipeline()
